using System;
using Microsoft.BizTalk.BaseFunctoids;
using System.Text;
using System.Reflection;
using System.Globalization;

namespace QueFunctoids
{
	/// <summary>
	/// Sample functoid (inline script version)
	/// </summary>
	public class InlineRoyaltyFunctoid : BaseFunctoid
	{
		public InlineRoyaltyFunctoid() : base() 
		{
			this.ID = 6046; // MS reserves 100 - 4000, suggests anything above 6000 for custom functoids
			
			// point to the resource assembly to use
			SetupResourceAssembly("QueFunctoids.QueFunctoidResources", Assembly.GetExecutingAssembly());
			
			// Set up basic resource information
			SetName("IDS_INLINEFUNCTOID_NAME");
			SetTooltip("IDS_INLINEFUNCTOID_TOOLTIP");
			SetDescription("IDS_INLINEFUNCTOID_DESCRIPTION");
			SetBitmap("IDB_INLINEFUNCTOID_BITMAP");

         this.SetMinParams(3);
         this.SetMaxParams(3);

			// Point to the function that provides this functoid's script source
			SetScriptBuffer(ScriptType.CSharp, ScriptSource());
			
			// Establish toolbox category and connection information
			this.Category = FunctoidCategory.Math;
			this.OutputConnectionType = ConnectionType.AllExceptRecord;			
			// Indicate needed global function
			this.RequiredGlobalHelperFunctions = InlineGlobalHelperFunction.IsNumeric;

			AddInputConnectionType((~ConnectionType.FunctoidCumulative)&(~ConnectionType.FunctoidDateTime)&ConnectionType.AllExceptRecord);
		}

		private string ScriptSource()
		{
			StringBuilder source = new StringBuilder();

			source.Append("public string CalcRoyalty(string rate, string price, string qty)\n");
			source.Append("{\n");
			source.Append("	decimal dRate, dPrice, dRetVal;\n");
			source.Append("	int iQty;\n");
			source.Append("	if (IsNumeric(rate) && IsNumeric(price) && IsNumeric(qty))\n");
			source.Append("	{\n");
			source.Append("		dRate = Convert.ToDecimal(rate, System.Globalization.CultureInfo.InvariantCulture);\n");
			source.Append("		dPrice = Convert.ToDecimal(price, System.Globalization.CultureInfo.InvariantCulture);\n");
			source.Append("		iQty = Convert.ToInt32(qty, System.Globalization.CultureInfo.InvariantCulture);\n");
			source.Append("		dRetVal = dRate/100 * dPrice * (decimal)iQty;\n");
			source.Append("		return dRetVal.ToString();\n");
			source.Append("	}\n");	
			source.Append("	else\n");
			source.Append("		return \"0.0\";\n");
			source.Append("}\n");

			return source.ToString();
		}
	}
	/// <summary>
	/// Sample functoid (global assembly cache version
	/// </summary>
	public class GlobalRoyaltyFunctoid : BaseFunctoid
	{
		public GlobalRoyaltyFunctoid() : base()
		{
			this.ID = 6045;
			
			// point to the resource assembly to use
			SetupResourceAssembly("QueFunctoids.QueFunctoidResources", Assembly.GetExecutingAssembly());
			
			// Set up basic resource information
			SetName("IDS_CACHEFUNCTOID_NAME");
			SetTooltip("IDS_CACHEFUNCTOID_TOOLTIP");
			SetDescription("IDS_CACHEFUNCTOID_DESCRIPTION");
			SetBitmap("IDB_CACHEFUNCTOID_BITMAP");

         this.SetMinParams(3);
         this.SetMaxParams(3);
			// Point to the function that implements this functoid's behavior
			// Params are assembly, class (fully qualified), method
			SetExternalFunctionName(GetType().Assembly.FullName, "QueFunctoids.GlobalRoyaltyFunctoid", "CalcRoyalty");

			// Establish toolbox category and connection information
			this.Category = FunctoidCategory.Math;
			this.OutputConnectionType = ConnectionType.AllExceptRecord;
			AddInputConnectionType(ConnectionType.AllExceptRecord);

		}

		public string CalcRoyalty(string rate, string price, string qty)
		{
			decimal dRate, dPrice, dRetVal;
			int iQty;

			if (IsNumeric(rate) && IsNumeric(price) && IsNumeric(qty))
			{
				dRate = Convert.ToDecimal(rate, System.Globalization.CultureInfo.InvariantCulture);
				dPrice = Convert.ToDecimal(price, System.Globalization.CultureInfo.InvariantCulture);
				iQty = Convert.ToInt32(qty, System.Globalization.CultureInfo.InvariantCulture);

				dRetVal = dRate/100 * dPrice * (decimal)iQty;
				return dRetVal.ToString();
			}
			else
				return "0.0";
		}

	}
}
